import { Ref, ref } from 'vue'
import { callApi, QueryParams } from '@/composables/engine/callApi'
import { start, stop } from '@/composables/engine/startStop'
import { fetchUpdate } from '@/composables/engine/fetchUpdate'
import { loadQuotes, getQuotesRef, deleteQuote, saveQuote } from '@/composables/engine/quotes'
import type { Quotes } from '@/composables/engine.d'

interface EngineContext {
    isActive: boolean
    shouldStop: boolean
    version: number
    lastFetched: Array<Date>
    isConnected: Ref<boolean>
    retry: Ref<number>
    callApi: (url: string, queryParams?: QueryParams) => Promise<unknown>
    fetchUpdate: () => Promise<void>
    loadQuotes: () => Promise<void>
}

export interface useEngine {
    isConnected: Ref<boolean>
    retry: Ref<number>
    start(): void
    stop(): void
    fetchUpdate(): void
    loadQuotes(): Promise<void>
    getQuotesRef(): Ref<Quotes>
    createQuote(quote: string): Promise<void>
    saveQuote(id: string, quote: string): Promise<void>
    deleteQuote(id: string): Promise<void>
}

export default (): useEngine => {
    const now = new Date(0)
    const context: EngineContext = {
        lastFetched: [now, now, now, now, now],
        isActive: false,
        shouldStop: false,
        version: -1,
        isConnected: ref(false),
        retry: ref(0),
        callApi,
        fetchUpdate,
        loadQuotes,
    }

    return {
        isConnected: context.isConnected,
        retry: context.retry,
        start: () => start.apply(context),
        stop: () => stop.apply(context),
        fetchUpdate: () => fetchUpdate.apply(context),
        loadQuotes: () => loadQuotes.apply(context),
        getQuotesRef: () => getQuotesRef.apply(context),
        createQuote: (quote: string) => saveQuote.apply(context, [':new:', quote]),
        saveQuote: (id: string, quote: string) => saveQuote.apply(context, [id, quote]),
        deleteQuote: (id) => deleteQuote.apply(context, [id]),
    }
}